#!/usr/bin/perl -w

# This file is a proof of concept logger for the RuoShui 86E Digital
# Multimeter, which is based on the ES51932 chipset.
#
# ES51932_log.pl is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# (c) 2021 Henk Vergonet

use strict;
use feature "switch";
use Device::SerialPort qw (:STAT);

my %conf = (
	port => '/dev/ttyUSB0',
	baud => 19200,
	databits => 7,
	stopbits => 1,
	parity => 'odd',
);

$| = 1;		# don't let Perl buffer I/O

################################################################################
#
# Serial Interface Setup
#
my $link = new Device::SerialPort($conf{port}) || die "new: $!\n";
$link->baudrate($conf{baud})	|| die 'fail setting baudrate, try -b option';
$link->parity($conf{parity})	|| die 'fail setting parity';
$link->databits($conf{databits})|| die 'fail setting databits';
$link->stopbits($conf{stopbits})|| die 'fail setting stopbits';
$link->handshake('none')	|| die 'fail setting handshake';
$link->datatype('raw')		|| die 'fail setting datatype';
$link->write_settings		|| die 'could not write settings';
$link->read_char_time(0);	# don't wait for each character
$link->read_const_time(250);	# 256*9/baud seconds per unfulfilled "read" call

################################################################################
# Main
################################################################################

while (1) {
        my ($cnt, $raw) = $link->read(128);
	if($cnt>0) {
		my ($r, $d, $f, $s, $o1, $o2, $o3, $o4) = unpack('CA5C6', $raw);
		my $exp = $r & 7;
		my $u;
		given($f & 15) {
			when(0) { $u = 'A'; $exp = -3; }	# 22 A
			when(1) { $u = 'V diode'; $exp = -4; }	# Diode
			when(2) { 				# Frequency
				if($s & 8) {
					$u = '%'; $exp = -1;
				} else {
					$u = 'Hz';
					$exp -= ($exp < 2 ? 2 : 3);
				} }
			when(3) { $u = 'Ω'; $exp -= 2; }	# Ohm
			when(4) { $u = '°C'; $exp = -1; }	# Temperature
			when(5) { $u = 'Ω cont.'; $exp = -2; }	# Continuity
			when(6) { $u = 'F'; $exp -= 12; }	# Capacitance
			when(11) { $u = 'V';			# Volt
				   $exp -= ($exp < 4 ? 4 : 9); }
			when(13) { $u = 'A'; $exp -= 8; }	# µA
			when(15) { $u = 'A'; $exp -= 6; }	# mA
			when(14) { $u = 'ADP'.(4-$exp); $exp = 0; }	# mA
		}
		if($s & 1) {
			$d = 'OL';
		} else {
			$d = '-'.$d if $s & 4;
			$d = "${d}e${exp}";
		}
		$u = $u.($o3 & 4 ? ' AC' : ' DC') if $u eq 'A' || $u eq 'V';
		my $flags = '';
		$flags .= ' auto' if $o3 & 2;
		#my @lpf = ('full','10kHz','1kHz','off'); 
		#$flags .= ' lpf('.$lpf[$o4 & 3].')';
		$flags .= ' lowbat' if $s & 2;

		print "$d $u$flags\n" ;
	}
}
$link->close;

